/* Client_BSD_Communicator.h defines sockets for TCP clients.
 *
 * Author: Joel Adams
 * Date: November 1998
 * Version: 0.9
 *
 * Modified by Ryan VanderBijl
 * Date: May/June 2000
 */

#ifndef CLIENT_BSD_COMMUNICATOR
#define CLIENT_BSD_COMMUNICATOR

#include "BSD_Communicator.h"
#include "Host.h"
#include "ClientSocketAddress.h"
using namespace std;


class Client_BSD_Communicator : public BSD_Communicator
{
 public:
  Client_BSD_Communicator();
  virtual ~Client_BSD_Communicator();
  Client_BSD_Communicator(const Client_BSD_Communicator &);

  void open(Service * service, const Host & host);
  void close();

  Host & hostInfo();
  virtual string className() const;
  Client_BSD_Communicator &operator=(const Client_BSD_Communicator &);

 protected:
  virtual void connect();

 private:
  Host myRemoteHost;
};

/*****************************************************************
 * Copy Construtcor
 * PRE: comm is an object which inherits from Client_BSD_Communicator
 * POST: data members are equal to comm's data memebers.
 *****************************************************************/
inline Client_BSD_Communicator::Client_BSD_Communicator(const Client_BSD_Communicator & comm)
    : BSD_Communicator(comm)
{
    this->myRemoteHost = comm.myRemoteHost;
}

/*****************************************************************
 * operator= - implements the assignment operator
 * PRE: comm is an object which inherits from Client_BSD_Communicator
 * POST: data members are equal to comm's data memebers.
 *****************************************************************/
inline Client_BSD_Communicator &Client_BSD_Communicator::operator=(const Client_BSD_Communicator & comm)
{
    BSD_Communicator::operator=(comm);
    this->myRemoteHost = comm.myRemoteHost;
    return *this;
}


/*****************************************************************
 * Default constructor (nothing to do).
 *****************************************************************/

inline Client_BSD_Communicator::Client_BSD_Communicator()
  : BSD_Communicator()
{}

/*****************************************************************
 * close the socket
 * (supplied for completeness, should be optimized away)
 * POST: the socket has been closed.
 *****************************************************************/

inline void Client_BSD_Communicator::close()
{
  BSD_Communicator::stop();
}


/*********************************************************
 * Destructor 
 * POST: The socket has been closed.
 *********************************************************/
inline Client_BSD_Communicator::~Client_BSD_Communicator()
{
  if (isOn())
    close();
}


/*****************************************************************
 * Host accessor.
 * POST: RV == myRemoteHost.
 *****************************************************************/
inline Host & Client_BSD_Communicator::hostInfo()
{
  return myRemoteHost;
}


/*********************************************************
 * className accessor
 * POST: RV == Client_BSD_Communicator.
 *********************************************************/

inline string Client_BSD_Communicator::className() const
{
  return "Client_BSD_Communicator";
}

#endif
