/* Client_TLI_Communicator.h defines sockets for TCP clients.
 *
 * Author: Joel Adams
 * Date: November 1998
 * Version: 0.9
 *
 * Modified by Ryan VanderBijl
 * Date: May/June 2000
 */

#ifndef CLIENT_TLI_COMMUNICATOR
#define CLIENT_TLI_COMMUNICATOR

#include "TLI_Communicator.h"
#include "Host.h"
#include "ClientSocketAddress.h"
using namespace std;

class Client_TLI_Communicator : public TLI_Communicator
{
 public:
  Client_TLI_Communicator();
  virtual ~Client_TLI_Communicator();
  Client_TLI_Communicator(const Client_TLI_Communicator &);

  void open(Service * service, const Host & host);
  void close();

  virtual string className() const;
  Client_TLI_Communicator &operator=(const Client_TLI_Communicator &);

 protected:
  virtual void connect() = 0;

 private:
};



/*****************************************************************
 * Copy Construtcor
 * PRE: comm is an object which inherits from Client_TLI_Communicator
 * POST: data members are equal to comm's data memebers.
 *****************************************************************/
inline Client_TLI_Communicator::Client_TLI_Communicator(const Client_TLI_Communicator & comm)
    : TLI_Communicator(comm)
{
}

/*****************************************************************
 * Default constructor (nothing to do).
 *****************************************************************/

inline Client_TLI_Communicator::Client_TLI_Communicator()
  : TLI_Communicator()
{}

/*****************************************************************
 * close the socket
 * (supplied for completeness, should be optimized away)
 * POST: the socket has been closed.
 *****************************************************************/

inline void Client_TLI_Communicator::close()
{
  TLI_Communicator::stop();
}


/*********************************************************
 * Destructor 
 * POST: The socket has been closed.
 *********************************************************/

inline Client_TLI_Communicator::~Client_TLI_Communicator()
{
  if (isOn())
    close();
}


/*********************************************************
 * className accessor
 * POST: RV == Client_TLI_Communicator.
 *********************************************************/

inline string Client_TLI_Communicator::className() const
{
  return "Client_TLI_Communicator";
}

#endif
