/* CommunicatorUser.h provides a base class for the Client & Server classes.
 *
 * Author: Joel Adams
 * Date: December 1998
 * Version: 0.9
 */

#ifndef TRANSPORTER_USER
#define TRANSPORTER_USER

#include "NetworkObject.h"
using namespace std;

template<class T>           // T is the Communicator type
class CommunicatorUser : public NetworkObject
{
 public:
  CommunicatorUser();
  virtual ~CommunicatorUser();

  int send(const string & msg);
  int receive(string & msg);

  T * communicatorInfo();

  virtual void print(ostream & out) const;
  virtual int compare(const NetworkObject & netObj) const;
  virtual string className() const;

 protected:
  virtual void setCommunicator(T * communicator);

 private:
  T * myCommunicator;
};

template<class T>
inline CommunicatorUser<T>::CommunicatorUser()
{
  myCommunicator = NULL;   // leave initialization to derived classes
}


template<class T>
inline CommunicatorUser<T>::~CommunicatorUser()
{
  delete myCommunicator;
}

template<class T>
inline T * CommunicatorUser<T>::communicatorInfo()
{
  return myCommunicator;
}


template<class T>
inline void CommunicatorUser<T>::setCommunicator(T * communicator)
{
  if (myCommunicator != NULL)
    delete myCommunicator;

  myCommunicator = communicator;
}

template<class T>
inline int CommunicatorUser<T>::send(const string & msg)
{
  return myCommunicator->send(msg);
}


template<class T>
inline int CommunicatorUser<T>::receive(string & msg)
{
  return myCommunicator->receive(msg);
}


template<class T>
inline string CommunicatorUser<T>::className() const
{
  return "CommunicatorUser<T>";
}


template<class T>
inline void CommunicatorUser<T>::print(ostream & out) const
{
  out << className() << " using " << (*myCommunicator);
}

template<class T>
inline int CommunicatorUser<T>::compare(const NetworkObject & netObj) const
{
  const CommunicatorUser<T> * ptr = 
    dynamic_cast<const CommunicatorUser<T> *>(&netObj);

  if (!ptr)
    fatal("CommunicatorUser<T>::compare(): Unable to compare %s and %s",
	  className().c_str(), netObj.className().c_str());

  if ((*myCommunicator) == (*(ptr->myCommunicator)))
    return 0;
  else
    return 1;
}

#endif
