/* IterativeServer.h defines an iterative server behavior template.
 *
 * Author: Joel Adams
 * Date: December 1998
 * Version: 0.9
 */

#ifndef ITERATIVE_SERVER
#define ITERATIVE_SERVER

#include "Server.h"
using namespace std;

template <class T>
class IterativeServer : public Server<T>
{
 public:
  IterativeServer(const string & serviceName, const unsigned short port);
  virtual ~IterativeServer();

  virtual void run();

  virtual string className() const;

 protected:

 private:

};


/*******************************************************************
 * Constructor.
 * PRE: serviceName is the name of the service I offer
 *       port is the port at which it is offered.
 * POST: I am ready to run.
 ******************************************************************/

template<class T>
inline IterativeServer<T>::IterativeServer(const string & serviceName, const unsigned short port)
  : Server<T>(serviceName, port)
{}

/******************************************************************
 * Destructor.
 * (nothing to do)
 ******************************************************************/
template<class T>
inline IterativeServer<T>::~IterativeServer()
{}


/*******************************************************
 * className accessor.
 * POST: RV == "IterativeServer<T>".
 *******************************************************/
template<class T>
inline string IterativeServer<T>::className() const
{
  return "IterativeServer<T>";
}


/******************************************************************
 * Service function (Iterative service).
 * PRE: I am running and ready to provide my service to a client.
 * POST: I have provided my service to a client.
 * Note: Overload in derived class if different behavior is needed.
 ******************************************************************/
template<class T>
inline void IterativeServer<T>::run()
{
  try
  {
      for(;;) //ever
      {
	  communicatorInfo()->acceptConnection();
	  interactWithClient();   // handle a client request
	  communicatorInfo()->closeConnection();
      }
  }
  catch(...)
  {
      warning("Server shutting down\n");
      return;
  }
}
#endif
