/* NetworkAddress.h defines an abstract interface for network addresses
 *  defined by a host and port number.
 *
 * Author: Joel Adams.
 * Date: December 1998
 * Version: 0.9
 */

#ifndef NETWORK_ADDRESS
#define NETWORK_ADDRESS

#include "NetworkObject.h"
#include "Host.h"
using namespace std;

class NetworkAddress: public NetworkObject
{
 public:
  NetworkAddress();
  NetworkAddress(const Host & host, const unsigned short port);
  NetworkAddress(const NetworkAddress & );

  virtual ~NetworkAddress();

  string hostName() const;

  virtual unsigned addressNBO() const;
  virtual string addressString() const;
  virtual unsigned short addressclassName() const;

  virtual unsigned short portNBO() const;
  virtual unsigned short portHBO() const;

  virtual string className() const;
  virtual void print(ostream & out) const;
  virtual int compare(const NetworkObject & netObj) const;
  NetworkAddress & operator=(const NetworkAddress &);

 protected:
  void setHost(const Host & host);
  void setPort(const unsigned short port);

 private:
  Host myHost;
  unsigned myPort;
};


/*****************************************************************
 * Copy Constructor
 * PRE: addr is an object which inherits from NetworkAddress
 * POST: data members are equal to addr's data memebers.
 *****************************************************************/
inline NetworkAddress::NetworkAddress(const NetworkAddress & addr)
{
   this->myHost = addr.myHost;
   this->myPort = addr.myPort;
}

/*****************************************************************
 * operator= - implements the assignment operator
 * PRE: addr is an object which inherits from NetworkAddress
 * POST: data members are equal to addr's data memebers.
 *****************************************************************/
inline NetworkAddress & NetworkAddress::operator=(const NetworkAddress &addr)
{
   this->myHost = addr.myHost;
   this->myPort = addr.myPort;
   return *this;
}

inline NetworkAddress::NetworkAddress()
{
  myPort = 0;
}

inline NetworkAddress::NetworkAddress(const Host & host,
				      const unsigned short port)
{
  myHost = host;
  myPort = htons(port);
}


inline NetworkAddress::~NetworkAddress()
{}


inline string NetworkAddress::hostName() const
{
  return myHost.name();
}

inline unsigned NetworkAddress::addressNBO() const
{
  return myHost.addressNBO();
}

inline string NetworkAddress::addressString() const
{
  return myHost.addressString();
}


inline unsigned short NetworkAddress::addressclassName() const
{
  return AF_INET;
}


inline unsigned short NetworkAddress::portNBO() const
{
  return myPort;
}


inline unsigned short NetworkAddress::portHBO() const
{
  return ntohs(myPort);
}


inline void NetworkAddress::setHost(const Host & host)
{
  myHost = host;
}

/* Port mutator
 * PRE: port is a valid machine port in network byte order
 * POST: myPort == port.
 */

inline void NetworkAddress::setPort(const unsigned short port)
{
  myPort = port;
}


inline string NetworkAddress::className() const
{
  return "NetworkAddress";
}



inline void NetworkAddress::print(ostream & out) const
{
  out << "NetworkAddress: " << myHost
      << ", port: " << portHBO();
}


#endif
