/* Proto.h declares the protocol interface for
 *  services using the TCP or UDP protocols.
 *
 * SHOULD be Protocol.h but MSVC++ 6.0 (at the least) had this nice
 *  system include file called protocol.h and it causes problems and
 *  blah blah blah. So we decided to change this file name simply
 *  to save ourselves from problems.
 *
 * Author: Joel C. Adams
 * Date: October, 1998.
 * Version 0.9
 */

#ifndef PROTOCOL
#define PROTOCOL

#ifndef WIN32
#include <netdb.h>                   // getprotobyname(), protoent
#else
#include <winsock2.h>
#endif /* WIN32 */

#include <string>                    // ANSI C++ string
#include "NetworkObject.h"        // base class
using namespace std;

class Protocol : public NetworkObject
{
 public:
  Protocol(const string & name);
  Protocol(const unsigned short number);
  virtual ~Protocol();
  Protocol(const Protocol & proto); 

  string name() const;
  short number() const;

  virtual void print(ostream & out) const;
  virtual int compare(const NetworkObject & protocol) const;
  virtual string className() const;
  Protocol& operator=(const Protocol & proto); 

  virtual int BSD_Info() const = 0;
  virtual char * TLI_Info() const = 0;

 protected:
  Protocol();

 private:
  string myName;
  short  myNumber;
};

/*****************************************************************
 * Copy Constructor
 * PRE: proto is an object which inherits from Protocol
 * POST: data members are equal to proto's data memebers.
 *****************************************************************/
inline Protocol::Protocol(const Protocol & proto)
{
   this->myName = proto.myName;
   this->myNumber = proto.myNumber;
}

/*****************************************************************
 * operator= - implements the assignment operator
 * PRE: proto is an object which inherits from Protocol
 * POST: data members are equal to proto's data memebers.
 *****************************************************************/
inline Protocol& Protocol::operator=(const Protocol & proto)
{
   this->myName = proto.myName;
   this->myNumber = proto.myNumber;
   return *this;
}

/************************************************
 * Default constructor (not much to do)
 ************************************************/

inline Protocol::Protocol()
{}


/********************************************************
 * Destructor (nothing to do)
 ********************************************************/

inline Protocol::~Protocol()
{}


/****************************************************
 * Extract the name of the protocol (e.g., "tcp")
 * POST: RV == myName
 ****************************************************/

inline string Protocol::name() const
{
  return myName;
}

/****************************************************
 * Extract the number of the protocol (e.g., 6 for tcp)
 * POST: RV == myNumber.
 ****************************************************/

inline short Protocol::number() const
{
  return myNumber;
}



/********************************************************
 * Print description of the class 
 * (Polymorphic, called by operator<<).
 * PRE: out is an open ostream.
 * POST: RV == my protocol number and name.
 ********************************************************/

inline void Protocol::print(ostream & out) const
{
  out << "Protocol " << myNumber
      << ": (" << myName << ')';
}

/********************************************************
 * class family
 * POST: RV == "Protocol".
 ********************************************************/

inline string Protocol::className() const
{
  return "Protocol";
}

#endif
