/* Server_Winsock_Communicator.h defines a Tranporter for TCP/IP servers
 *    using Winsock sockets.
 *
 * Author: Joel Adams
 * Date: November 1998
 * Version: 0.9
 *
 * Modified by: Ryan VanderBijl
 * Date: May/June 2000
 */

#ifndef SERVER_Winsock_TRANSPORTER
#define SERVER_Winsock_TRANSPORTER

#include "Winsock_Communicator.h"
#include "Service.h"
#include "ServerSocketAddress.h"
using namespace std;

class Server_Winsock_Communicator : public Winsock_Communicator
{
 public:
  Server_Winsock_Communicator();
  virtual ~Server_Winsock_Communicator();
  Server_Winsock_Communicator(const Server_Winsock_Communicator & comm);

  void open(Service * service);
  void close();

  virtual void acceptConnection() = 0;
  virtual void closeConnection() = 0;
  virtual bool hasConnection() const;

  virtual string className() const;
  Server_Winsock_Communicator & 
      operator=(const Server_Winsock_Communicator & comm);

  SocketAddress & client();

 protected:
  void bind();
  void setConnected(bool value);
  void setClient(const SocketAddress & client);

 private:
  SocketAddress myClient;
  bool connected;
};



/*****************************************************************
 * Default Constructor -- sets  connecttion status to false
 *****************************************************************/
inline Server_Winsock_Communicator::Server_Winsock_Communicator()
  : Winsock_Communicator()
{
  connected = false;
}


/*****************************************************************
 * bind socket to port
 * PRE: I have been initialized with a Service or port.
 * POST: I have been bound to the port.
 *****************************************************************/
inline void Server_Winsock_Communicator::bind()
{
  int notOk = ::bind(getDescriptor(),
	   Winsock_Communicator::addressInfo()->addressInfoOldFormat(),
	   Winsock_Communicator::addressInfo()->size());
  if (notOk)
    fatal("Server_Winsock_Communicator::bind(): unable to bind socket");
}


/*****************************************************************
 * open with Service
 * PRE: serviceName holds the name of a valid TCP Service.
 * POST: myServiceInfo == service.
 *****************************************************************/
inline void Server_Winsock_Communicator::open(Service * service)
{
  Winsock_Communicator::start(service);
  Winsock_Communicator::setAddressInfo(new ServerSocketAddress("localhost", 
							  *service));
  bind();
}


/****************************************************************
 * Connection status indicator.
 * POST: RV == true iff I have accepted a connection.
 ****************************************************************/
inline bool Server_Winsock_Communicator::hasConnection() const
{
  return connected;
}

/****************************************************************
 * Connection mutator
 * sets the connection status to value
 ****************************************************************/
inline void Server_Winsock_Communicator::setConnected(bool value)
{
  connected = value;
}



/*****************************************************************
 * close the socket
 * (supplied for completeness, should be optimized away)
 * POST: the socket has been closed.
 *****************************************************************/
inline void Server_Winsock_Communicator::close()
{
  Winsock_Communicator::stop();
}


/*********************************************************
 * className accessor
 * POST: RV == Server_Winsock_Communicator.
 *********************************************************/
inline string Server_Winsock_Communicator::className() const
{
  return "Server_Winsock_Communicator";
}

/*********************************************************
 * Destructor 
 * POST: The socket has been closed.
 *********************************************************/
inline Server_Winsock_Communicator::~Server_Winsock_Communicator()
{
  if (isOn())
    close();
}


/******************************************************
 * client accessor.
 *   May be outdated if there is no current connection.
 *   (is set in acceptConnection, but not unset anywhere)
 * POST: RV == myClient
 ******************************************************/
inline SocketAddress & Server_Winsock_Communicator::client()
{
  return myClient;
}

/******************************************************
 * setClient -- sets myClient == cleint
 * PRE: sender is the current client
 ******************************************************/
inline void Server_Winsock_Communicator::setClient(const SocketAddress & client)
{
  myClient = client;
}
 

#endif
