/*****************************************************************
 * TLI_UDP_SendRecv.h
 *  This is a "helper" class for TLI UDP classes. This implements
 *  the actual send/recv calls, so that we do not need to define
 *  them multiple times.
 *
 * It should have just had to be a parent class to the
 * UDP_{Client|Se4rver}_TLI_Communicator classes, but I seems
 * the multiple inheritance/polymorphism  code in gcc 2.95.1
 * is a bit broken. Both the actual call to these functions weren't
 * being found, neither were functions like getDescriptor() when
 * called from in here.
 *
 * Currently the classes which inherrit this class call these
 * functions from their send/receive, and pass the correct
 * info needed.
 *
 * Author: Ryan VanderBijl
 * Date:   June 2000
 */

#ifndef TLI_UDP_SEND_RECV
#define TLI_UDP_SEND_RECV

#include <string>
using namespace std;

class TLI_UDP_SendRecv {
public:
    //nothing that the end user can use
protected:
    typedef struct t_unitdata *pud_t;
    int send(int, pud_t, const string & msg);
    int receive(int, pud_t, string & msg);
private:
    //nothing to store
};


/************
 * send -- send the message msg, using datagram pud, on 
 *         the TLI endpoint d.
 * PRE: d is an allocated endpoint,
 *      pud is an initialized datagram, with the server address stored
 *      msg contains the message to be sent.
 * POST: the message has been sent to the server, and the value of
 *       t_sndudata is returned: 0 if okay, -1 if error (t_errno is set)
 ************/
inline int TLI_UDP_SendRecv::send(int d, pud_t pud,const string & msg) 
{
   strcpy(pud->udata.buf, msg.c_str());
   pud->udata.len = msg.length();

   return t_sndudata(d, pud);
}


/************
 * receive -- receive a message msg, using datagram pud, on
 *         the TLI endpoint d.
 * PRE: d is an allocated endpoint,
 *      pud is an initialized datagram, with the server address stored
 * POST: the message has been received from the server, and the value of
 *       t_rcvudata is returned: 0 if okay, -1 if error (t_errno is set)
 ************/
inline int TLI_UDP_SendRecv::receive(int d, pud_t pud, string & msg) 
{
   int flags,cc;
   cc = t_rcvudata(d, pud, &flags);
   if (flags & T_MORE) {
      cout << "IT WANTS MORE!! " << endl;
      exit(1);
   }

   if (pud->udata.len == pud->udata.maxlen) {
      --pud->udata.len;
      --cc;
   }

   pud->udata.buf[pud->udata.len] = '\0';
   msg = pud->udata.buf;
   return cc;
}

#endif
