#include "Server_TLI_Communicator.h"

/*****************************************************************
 * bind socket to port
 * PRE: I have been initialized with a Service or port.
 * POST: I have been bound to the port.
 *****************************************************************/
void Server_TLI_Communicator::bind()
{
   struct t_bind *req, *ret;
   req = (struct t_bind*)t_alloc(getDescriptor(), T_BIND, T_ADDR);
   ret = (struct t_bind*)t_alloc(getDescriptor(), T_BIND, T_ADDR);

   if ((req == 0) || (ret == 0)) 
       fatal("couldn't t_alloc.\n");

   memcpy(req->addr.buf, addressInfo()->addressInfo(),
	      sizeof(struct sockaddr_in));
   req->addr.len = sizeof(struct sockaddr_in);
   req->qlen = this->queueLength();

   if (t_bind(getDescriptor(), req, ret) < 0)
       fatal("t_bind didn't work.\n");
  
   struct sockaddr_in rsin;  //return sockaddress info
   memcpy(&rsin, ret->addr.buf, sizeof(rsin));
   
   if (addressInfo()->portHBO() != rsin.sin_port) 
       fatal("Couldn't assign request address\n");

}


/*****************************************************************
 * open with Service
 * PRE: serviceName holds the name of a valid TCP Service.
 * POST: myServiceInfo == service.
 *****************************************************************/
void Server_TLI_Communicator::open(Service * service)
{
  TLI_Communicator::start(service);
  TLI_Communicator::setAddressInfo(new ServerSocketAddress("localhost", 
							  *service));
  bind();
}


/*****************************************************************
 * Copy constructor
 * PRE: comm is an object which inherits from Server_TLI_Communicator
 * POST: data members are equal to comm's data memebers.
 *****************************************************************/
Server_TLI_Communicator::Server_TLI_Communicator(const Server_TLI_Communicator & comm)
  : TLI_Communicator(comm)
{
   this->connected = comm.connected;
   this->myQueueLength = comm.myQueueLength;
   this->myClient = comm.myClient;
}


/*****************************************************************
 * operator= - implements the assignment operator
 * PRE: comm is an object which inherits from Server_TLI_Communicator
 * POST: data members are equal to comm's data memebers.
 *****************************************************************/
Server_TLI_Communicator & Server_TLI_Communicator::operator=(const Server_TLI_Communicator & comm)
{
   TLI_Communicator::operator=(comm);
   this->connected     = comm.connected;
   this->myQueueLength = comm.myQueueLength;
   this->myClient      = comm.myClient;
   return *this;
}
