#include "UDP_Client_TLI_Communicator.h"

/*****************************************************************
 * connect
 *   Doesn't really connect. However, this function does allocate
 *   the data/information needed for UDP to connect. Remember, UDP
 *   is connectionless!  After this messages can be sent.
 * PRE: a TLI endpoint has been allocated
 * POST: a datagram has been allocated. 
 *****************************************************************/
void UDP_Client_TLI_Communicator::connect()
{
    pud = (pud_t)t_alloc(getDescriptor(), T_UNITDATA, T_ADDR | T_UDATA);
    if (!pud) fatal("couldnt allocate pud:");

    // set up the datagram's server address, so we may send it messages
    memcpy(pud->addr.buf, addressInfo()->addressInfo(),
	   sizeof(struct sockaddr_in));
    pud->addr.len = sizeof(struct sockaddr_in);
}


/*****************************************************************
 * Copy Construtcor
 * PRE: comm is an object which inherits from UDP_Client_TLI_Communicator
 * POST: data members are equal to comm's data memebers.
 *****************************************************************/
UDP_Client_TLI_Communicator::UDP_Client_TLI_Communicator(const UDP_Client_TLI_Communicator & comm)
    : Client_TLI_Communicator(comm)
{
    if (pud) t_free((char*)pud, T_UNITDATA);
    pud = (pud_t)t_alloc(getDescriptor(), T_UNITDATA, T_ADDR | T_UDATA);

    if (!pud) fatal("couldnt allocate pud:");
    memcpy(pud, comm.pud, sizeof(struct t_unitdata));
}


/*****************************************************************
 * operator= - implements the assignment operator
 * PRE: comm is an object which inherits from UDP_Client_TLI_Communicator
 * POST: data members are equal to comm's data memebers.
 *****************************************************************/
UDP_Client_TLI_Communicator &UDP_Client_TLI_Communicator::operator=(const UDP_Client_TLI_Communicator & comm)
{
    Client_TLI_Communicator::operator=(comm);

    if (pud) t_free((char*)pud, T_UNITDATA);
    pud = (pud_t)t_alloc(getDescriptor(), T_UNITDATA, T_ADDR | T_UDATA);
    if (!pud) fatal("couldnt allocate pud:");
    memcpy(pud, comm.pud, sizeof(struct t_unitdata));

    return *this;
}
