/** CPlusPlusGenerator.java 
 *  @author  Joel Adams, Summer 2000.
 *  @author  Ryan VanderBijl, Summer 2000
 *  All rights reserved
 */
package cliser;

import java.io.*;                // BufferedReader
import java.awt.*;
import java.util.Date;
import javax.swing.JOptionPane;
import cliser.Generator;

/** A CPlusPlusGenerator is used by a given Cliser interface
 *  to generate C++ code for a client-server system,
 *  as specified by the parameters entered at the interface.
 *  <P>
 *  See the class constructor for the parameters.
 */
public class CPlusPlusGenerator extends Generator
{
    /** Constructor for use by a Cliser GUI.
     *  @param serviceName the name of the service
     *  @param port the port the service should use
     *  @param protocol "TCP" or "UDP"
     *  @param networkInterface "BSD" or "TLI" or "Winsock"
     *  @param parentFrame the Frame of my GUI
     * <P>
     * Precondition: <TT>serviceName</TT> is the name of the service (e.g., "ftp")<BR>
     *               && <TT>port</TT> is the port for this service<BR>
     *               && <TT>protocol</TT> is either "TCP" or "UDP"<BR>
     *               && <TT>networkInterface</TT> is either "BSD", "TLI", or "Winsock"<BR>
     *               && <TT>parentFrame</TT> refers to my parent GUI<BR>
     * Postcondition: my instance variables have been initialized<BR>
     *                 using the parameter values.
     */
    public CPlusPlusGenerator(String serviceName, int port,
			      String protocol, String networkInterface,
			      Frame parentFrame)
    {
	super(serviceName, port, protocol);

	myNetworkInterface = networkInterface;
	myParentFrame      = parentFrame;
    }

    /** Constructor for use by a CLI.
     *  @param serviceName the name of the service
     *  @param port the port the service should use
     *  @param protocol "TCP" or "UDP"
     *  @param networkInterface "BSD" or "TLI" or "Winsock"
     *  @param parentFrame the Frame of my GUI
     * <P>
     * Precondition: <TT>serviceName</TT> is the name of the service (e.g., "ftp")<BR>
     *               && <TT>port</TT> is the port for this service<BR>
     *               && <TT>protocol</TT> is either "TCP" or "UDP"<BR>
     *               && <TT>networkInterface</TT> is either "BSD", "TLI", or "Winsock"<BR>
     * Postcondition: my instance variables have been initialized<BR>
     *                 using the parameter values
     *               && <TT>myParentFrame</TT> == null.
     */
    public CPlusPlusGenerator(String serviceName, int port,
			      String protocol, String networkInterface)
    { this(serviceName, port, protocol, networkInterface, null); }



    /** Access my network interface information.
     *  @return myNetworkInterface
     */
    public final String getNetworkInterface()
    {
	return myNetworkInterface;
    }

    /** Generate C++ client-code using parameters given to my constructor.
     *  Postcondition: the source code has been generated for a client 
     *                  for myService, using myPort, myNetworkInterface,
     *                  and myProtocol
     */
    public void generateClient()
    {
	// set up some variables:
	String clientName = this.getService() + "_" + this.getProtocol()
			      + "_" + myNetworkInterface + "_Client";
        String commName   = this.getProtocol() + "_Client_" + 
			    myNetworkInterface + "_Communicator";
	String clientFileName = clientName + ".cpp";

	// open the file
	PrintWriter writer = null;
        try
        {
          writer = new PrintWriter( new FileWriter(clientFileName) );
        }
        catch (Exception e)
        {
            System.err.println("CPPGenerator.generateClient(): " + e);
            System.exit(1);
        }

	Date date = new Date();
	writer.println("/******************************************\n" +
		       " * " + clientFileName                   + "\n" +
		       " * Auto-generated by Cliser, " + date   + "\n" +
		       " *  except for the body of method:"     + "\n" +
		       " *   " + clientName+"::interactWithClient()\n" +
		       " *"                                     + "\n" +
		       " * See http://cs.calvin.edu/~cliser/ for "     +
		                                     "more details.\n" +
		       " */"                                    + "\n" +
		       ""                                       + "\n" +
		       "#include \"" + commName + ".h\""        + "\n" +
		       "#include \"Client.h\""                  + "\n" +
		       ""                                       + "\n" +
		       "class " + clientName + " : public Client<"     +
		                                      commName + ">\n" +
		       "{"                                      + "\n" +
		       "public:"                                + "\n" +
		       "  " + clientName + "(const unsigned short "    +
		                 "port, const string & remoteHost);\n" +
		       "  void interactWithServer();"           + "\n" +
		       "};"                                     + "\n" +
		       ""                                       + "\n" +
		       "void " + clientName+"::interactWithServer()\n" +
		       "{"                                      + "\n" +
		       "  // replace these comments with statements\n" +
		       "  // that define your client's behavior    \n" +
		       "}"                                      + "\n" +
		       ""                                       + "\n" +
		       clientName+"::" + clientName + "(const unsigned"+
		           " short port, const string & remoteHost)\n" +
		       "    : Client<" + commName +">(\""              +
		         this.getService() + "\", port, remoteHost)\n" +
		       "{"                                      + "\n" +
		       "  // optional:"                         + "\n" +
		       "  // replace these comments with statements\n" +
		       "  // that initialize your client"       + "\n" +
		       "}"                                      + "\n" +
		       ""                                       + "\n" +
		       ""                                       + "\n" +
		       "int main(int argc, char** argv)"        + "\n" +
		       "{"                                      + "\n" +
		       "  unsigned short port = "+this.getPort()+";\n" +
		       ""                                       + "\n" +
		       "  switch(argc) {"                       + "\n" +
		       "     default:"                          + "\n" +
		       "       cerr << \"Usage: \" << argv[0] << \""   +
		                     " <server> [<port>]\" << endl;\n" +
		       "       return -1;"                      + "\n" +
		       "     case 3:"                           + "\n" +
		       "       port = (unsigned short)atoi(argv[2]);\n"+
		       "       /* fall through */"              + "\n" +
		       "     case 2:"                           + "\n" +
		       "       " + clientName + " myClient(port, "     +
		                                         "argv[1]);\n" +
		       "       myClient.run();"                 + "\n" +
		       "  }"                                    + "\n" +
		       "  return 0;"                            + "\n" +
		       "}"                                      + "\n");

	// we're done, so let the user know.
	writer.close();
        if (myParentFrame != null)
            JOptionPane.showMessageDialog(myParentFrame,
			  "Client written to file: " + clientFileName,
			  "Client completed...",
			  JOptionPane.INFORMATION_MESSAGE);
        else
            System.out.println("CPPGenerator.generateClient(): "
                            + " Client completed and written to file: '"
                            + clientFileName + "'\n\n");

    }


    /** Generate C++ server code using constructor parameters
     *   and <TT>serverType</TT>.
     *  @param serverType "Iterative" or "Unix" or "POSIX"
     *  <P>
     *  Precondition: <TT>serverType</TT> is
     *                     "Iterative" for an iterative/single threaded server<BR>
     *                     "Unix"  for a Unix concurrent server<BR>
     *                     "POSIX" for a POSIX threaded concurrent server<BR>
     *  <P>
     *  Postcondition: the source code has been generated for a server
     *                  for myService, using myPort, myNetworkInterface,
     *                  and myProtocol
     */
    public void generateServer(String serverType)
    {
	// set up some variables:
	String serverName = this.getService() + "_" + serverType;

	if (serverType.equals("Unix")) {
	    serverName += "_Concurrent";
	    serverType += "Concurrent";
	} else if (serverType.equals("POSIX")) {
	    serverName += "_Concurrent";
	    serverType  = "PThreadsConcurrent";
	} 
	serverType += "Server";
	
	serverName += "_" + this.getProtocol() + "_" + 
		      myNetworkInterface + "_Server";
   
	String serverFileName = serverName + ".cpp";

        String threadName = this.getService() + "_" + 
			    this.getProtocol() + "_" +
			    myNetworkInterface + "_Thread";
        String commName   = this.getProtocol() + "_Server_" + 
			    myNetworkInterface + "_Communicator";

	String serverParentClass = serverType + "<" + commName;
	if (serverType.equals("PThreadsConcurrentServer")) {
	    serverParentClass +=  ", " + threadName;
	}
	serverParentClass += ">";


	// open the file
	PrintWriter writer = null;
        try
        {
          writer = new PrintWriter( new FileWriter(serverFileName) );
        }
        catch (Exception e)
        {
            System.err.println("CPPGenerator.generateServer(): " + e);
            System.exit(1);
        }

	// The default Header
	Date date = new Date();
	writer.println("/******************************************\n" +
		       " * " + serverFileName                   + "\n" +
		       " * Auto-generated by Cliser, " + date   + "\n" +
		       " *  except for the body of method:"     + "\n" +
		       " *   " + serverName + "::run()"         + "\n" +
		       " *"                                     + "\n" +
		       " * See http://cs.calvin.edu/~cliser/ for "     +
		                                     "more details.\n" +
		       " */"                                    + "\n" +
		       ""                                       + "\n" +
		       "#include \"" + commName + ".h\""        + "\n" +
		       "#include \"" + serverType + ".h\""      + "\n");

	if (serverType.equals("PThreadsConcurrentServer")) {
	    writer.println(
		       "#include \"ServiceThread.h\""           + "\n" +
		       ""                                       + "\n" +
		       "class " + threadName + " : public "            +
			      "ServiceThread<" + commName + ">" + "\n" +
		       "{"                                      + "\n" +
		       "public:"                                + "\n" +
		       "  " + threadName + "();"                + "\n" +
		       "  void interactWithClient();"           + "\n" +
		       "};"                                     + "\n" +
		       ""                                       + "\n" +
		       "void " + threadName+"::interactWithClient()\n" +
		       "{"                                      + "\n" +
		       "  // replace these comments with statements\n" +
		       "  // that define your server's behavior    \n" +
		       "}"                                      + "\n" +
		       ""                                       + "\n" +
		       threadName + "::" + threadName + "()"    + "\n" +
		            "  :  ServiceThread<" + commName + ">()\n" +
		       "{"                                      + "\n" +
		       "  // optional:"                         + "\n" +
		       "  // replace these comments with statements\n" +
		       "  // that initialize your server thread"+ "\n" +
		       "}"                                      + "\n");
	}


	writer.println(""                                       + "\n" +
	               "class " + serverName                    + "\n" +
		       "    : public " + serverParentClass      + "\n" +
		       "{"                                      + "\n" +
		       "public:"                                + "\n" +
		       "  "+serverName+"(const unsigned short port);" );
		       

	if (!serverType.equals("PThreadsConcurrentServer")) {
	    writer.println("  void interactWithClient();");
	}
	writer.println("};\n");

	if (!serverType.equals("PThreadsConcurrentServer")) {
	   writer.println(
	               "void " + serverName + "::"                     +
					      "interactWithClient()\n" +
		       "{"                                      + "\n" +
		       "  // replace these comments with statements\n" +
		       "  // that define your server's behavior    \n" +
		       "}"                                      + "\n");
	}



        writer.println(""                                       + "\n" +
	               serverName + "::" + serverName + "(const "      +
		                              "unsigned short port)\n" +
		       "  : " + serverParentClass + "(\""              +
			             this.getService() + "\", port)\n" +
		       "{"                                      + "\n" +
		       "  // optional:"                         + "\n" +
		       "  // replace these comments with statements\n" +
		       "  // that initialize your server"       + "\n" +
		       "}"                                      + "\n" +
		       ""                                       + "\n" +
		       ""                                       + "\n" +
		       "int main(int argc, char** argv)"        + "\n" +
		       "{"                                      + "\n" +
		       "  unsigned short port = "+this.getPort()+";\n" +
		       ""                                       + "\n" +
		       "  switch(argc) {"                       + "\n" +
		       "     default:"                          + "\n" +
		       "       cerr << \"Usage: \" << argv[0] << \""   +
		                              " [<port>]\" << endl;\n" +
		       "       return -1;"                      + "\n" +
		       "     case 2:"                           + "\n" +
		       "       port = (unsigned short)atoi(argv[1]);\n"+
		       "       /* fall through */"              + "\n" +
		       "     case 1:"                           + "\n" +
		       "       " + serverName + " myServer(port);\n"   +
		       "       myServer.run();"                 + "\n" +
		       "  }"                                    + "\n" +
		       "  return 0;"                            + "\n" +
		       "}"                                      + "\n");

	// close the file, let the user know what we have done.
	writer.close();
        if (myParentFrame != null)
            JOptionPane.showMessageDialog(myParentFrame,
			  "Server written to file: " + serverFileName,
			  "Server completed...",
			  JOptionPane.INFORMATION_MESSAGE);
        else
            System.out.println("CPPGenerator.generateServer(): "
                            + " Server completed and written to file: '"
                            + serverFileName + "'\n\n");
    }




    /** Generate Makefile for C++ code. 
     *  @param buildClient true iff the user wants a client 
     *  @param buildServer true iff the user wants a server
     *  @param serverType "Iterative" or "Concurrent" (for Java)
     *                    "Iterative" or "Unix" or "POSIX" (for C++)
     *  <P>
     *  Precondition: (<TT>buildServer</TT> is true &&  mySourceLanguage is "Java")<BR>
     *                 --> (<TT>serverType</TT> is "Iterative"
     *                      || <TT>serverType</TT> is "Concurrent")<BR>
     *                && (<TT>buildServer</TT> is true && mySourceLanguage is "C++")<BR>
     *                 --> (<TT>serverType</TT> == "Iterative"<BR>
     *                      || <TT>serverType</TT> == "Unix"<BR>
     *                      || <TT>serverType</TT> == "POSIX").
     *  <P>
     *  Postcondition: A Makefile has been generated to compile
     *                 the user program.
     */
    public void generateMakefile(boolean buildClient,
				 boolean buildServer,
				 String serverType
				 )
    {
	if (!serverType.equals("Iterative")) {
	    serverType = serverType + "_" + "Concurrent";
	}

	// open the file
	PrintWriter writer = null;
        try
        {
          writer = new PrintWriter( new FileWriter("Makefile") );
        }
        catch (Exception e)
        {
            System.err.println("CPPGenerator.generateServer(): " + e);
            System.exit(1);
        }

	writer.println(
"###"                                                           + "\n" +
"# Cliser C++ User Makefile"                                    + "\n" +
"#   Configure the top part of the Makefile to suit your project.  \n" +
"###"                                                           + "\n" +
""                                                              + "\n" +
"## Service Name: (as entered into Cliser)"                     + "\n" +
"SERVICE_NAME = " + getService()                                + "\n" +
""                                                              + "\n" +
"## Which protocol are you using? Options: TCP or UDP"          + "\n" +
"PROTOCOL = " + getProtocol()                                   + "\n" +
""                                                              + "\n" +
"## Are you building a client and/or server?  Options: client server\n"+
"PROJECTS = " + (buildClient ? "client " : " ") +
	        (buildServer ? "server " : " ")                 + "\n" +
""                                                              + "\n" +
"### Set the target names for your code:"                       + "\n" +
"CLIENT_TARGET = $(SERVICE_NAME)"                               + "\n" +
"SERVER_TARGET = $(SERVICE_NAME)d  # 'd' for daemon"            + "\n" +
""                                                              + "\n" +
"## Which Abstraction are you using? Options: BSD or TLI"       + "\n" +
"ABSTRACTION = " + myNetworkInterface                           + "\n" +
""                                                              + "\n" +
"## Which type of server is it?"                                + "\n" +
"## Options: Iterative    Unix_Concurrent    POSIX_Concurrent"  + "\n" +
"SERVER_TYPE = " + serverType                                   + "\n" +
""                                                              + "\n" +
"####"                                                          + "\n" +
"# Compiling configuration:"                                    + "\n" +
"####"                                                          + "\n" +
""                                                              + "\n" +
"CLISER_DIR = /usr/local/cliser"                                + "\n" +
"INC_DIR    = $(CLISER_DIR)/include"                            + "\n" +
"LIB_DIR    = $(CLISER_DIR)/lib"                                + "\n" +
""                                                              + "\n" +
"## uncomment to include debugging symbols"                     + "\n" +
"#DEBUGGING = -g"                                               + "\n" +
"CC = g++"                                                      + "\n" +
""                                                              + "\n" +
"CFLAGS = $(DEBUGGING) -O2 -fPIC -Wall -ansi -pedantic -I$(INC_DIR)\n" +
"LFLAGS = -L$(LIB_DIR) -lcliser -lnsl -lsocket -lpthread"       + "\n" +
""                                                              + "\n" +
"###"                                                           + "\n" +
"# the rest should be fine"                                     + "\n" +
"###"                                                           + "\n" +
".PHONY: all client server"                                     + "\n" +
""                                                              + "\n" +
"all:  $(PROJECTS)"                                             + "\n" +
"client: $(CLIENT_TARGET)"                                      + "\n" +
"server: $(SERVER_TARGET)"                                      + "\n" +
""                                                              + "\n" +
"CLIENT_CODE = $(SERVICE_NAME)_$(PROTOCOL)_$(ABSTRACTION)_Client.cpp\n"+
"SERVER_CODE = $(SERVICE_NAME)_$(SERVER_TYPE)_$(PROTOCOL)_$(ABSTRACTION)_Server.cpp\n" +
""                                                              + "\n" +
"$(CLIENT_TARGET): $(CLIENT_CODE)"                              + "\n" +
"	$(CC) $(CFLAGS) $(CLIENT_CODE) -o $@ $(LFLAGS)"         + "\n" +
""                                                              + "\n" +
"$(SERVER_TARGET): $(SERVER_CODE)"                              + "\n" +
"	$(CC) $(CFLAGS) $(SERVER_CODE) -o $@ $(LFLAGS)"         
	);

	// close the file, let the user know what we have done.
	writer.close();
        if (myParentFrame != null)
            JOptionPane.showMessageDialog(myParentFrame,
			  "Makefile written to file: Makefile",
			  "Makefile completed...",
			  JOptionPane.INFORMATION_MESSAGE);
        else
            System.out.println("CPPGenerator.generateMakefile(): "
                           + "Makefile completed and written to file: '"
                           + "Makefile'\n\n");
    }


    private String myNetworkInterface;
    private Frame  myParentFrame;
}

