/** Generator.java 
 *  @author  Joel Adams, Spring 2000.
 *  All rights reserved
 */

package cliser;

abstract public class Generator extends Object
{
    /** constructor
     *  @params string serviceName
     *  @params int port
     *  @params string protocol
     * Precondition: serviceName is the name of the service (e.g., "ftp")
     *               && port is the port for this service
     *               && protocol is either "TCP" or "UDP"
     * Postcondition: my instance variables have been initialized
     *                 using the parameter values.
     */
    public Generator(String serviceName, int port, String protocol)
    {
	myServiceName = serviceName;
	myPort = port;
	myProtocol = protocol;
    }


    /** service accessor
     *  @return myServiceName
     */
    public final String getService() { return myServiceName; }


    /** port accessor
     *  @return myPort
     */
    public final int getPort() { return myPort; }


    /** protocol accessor
     *  @return myProtocol
     */
    public final String getProtocol() { return myProtocol; }


    /** client generator
     *  Postcondition: the source code has been generated for a client 
     *                  for myService, using myPort, myNetworkInterface,
     *                  and myProtocol
     */
    abstract public void generateClient();


    /** server generator
     *  @params: String concurrentType
     *  Precondition: (mySourceLanguage == "Java")
     *                 --> (concurrentType == "Iterative"
     *                      || concurrentType == "Concurrent")
     *                && (mySourceLanguage == "C++")
     *                 --> (concurrentType == "Iterative"
     *                      || concurrentType == "Unix"
     *                      || concurrentType == "POSIX").
     *  Postcondition: the source code has been generated for a server
     *                  for myService, using myPort, myNetworkInterface,
     *                  myProtocol, and concurrentType.
     */
    abstract public void generateServer(String concurrentType);



    /** Makefile generator
     *  @params: boolean buildClient, buildServer, String concurrentType
     *  Precondition: (buildClient == true || buildServer == true) 
     *                && (buildServer == true)
     *                && (mySourceLanguage == "Java")
     *                 --> (serverType == "Iterative"
     *                      || serverType == "Concurrent")
     *                && (buildServer == true)
     *                && (mySourceLanguage == "C++")
     *                 --> (serverType == "Iterative"
     *                      || serverType == "Unix"
     *                      || serverType == "POSIX").
     *  Postcondition: A Makefile has been generated to compile
     *                 the user program.
     */
    abstract public void generateMakefile(boolean buildClient, 
					  boolean buildServer,
					  String serverType
					  );

    private String mySourceLanguage, myServiceName, myProtocol;
    private int    myPort;
}

