/** IterativeTCPServer.java
 *  @author Joel C. Adams
 *  @version 1.0.  Copyright 2000.  All rights reserved.
 */

package cliser;

import cliser.TCPServer;

abstract public class IterativeTCPServer extends TCPServer
{
    /** explicit communicator
     *  @params service, a String naming my service
     *  @params port, an int specifying the port I will use
     * POST: my (inherited) instance variables have been initialized.
     */
    public IterativeTCPServer(String service, int port)
    {
	super(service, port);
    }


    /** definition of abstract CommunicatorUser.initCommunicator()
     *  PRE: myPort has been initialized.
     *  @return a new IterativeServerTCPCommunicator listening on myPort.
     */
    protected Communicator initCommunicator()
    {
	return new IterativeServerTCPCommunicator( this.getPort() );
    }

    /** default iterative server behavior
     *   (run forever, accepting connections and performing service).
     */
    public void run()
    {
        System.err.println("IterativeTCPServer: running on "
		   + this.getHost() + ":" + this.getPort());
	for (;;)
        {
	    this.acceptConnection();
    //            System.err.println("IterativeTCPServer.run(): "
    //			       + "connection accepted via "
    //                         + getCommunicator() );
	    this.interactWithClient();
    //            System.err.println("IterativeTCPServer.run(): "
    //			       + "service completed for "
    //                         + getCommunicator().getRemoteHost());
	}
    }

    /** method defined by subclass specifying server's interaction with client
     *  PRE: A connection has been accepted.
     *  POST: my service has been performed for that connection's client
     */
    abstract public void interactWithClient();
}
