/** JavaGenerator.java 
 *  @author  Joel Adams, Spring 2000.
 *  All rights reserved
 */
package cliser;

import java.io.*;                // BufferedReader, PrintWriter, ...
import java.awt.*;               // Frame, FileDialog, ...
import java.util.Date;
import javax.swing.JOptionPane;
import cliser.Generator;

public class JavaGenerator extends Generator
{
    /** constructor for graphical user interfaces
     *  @params string serviceName
     *  @params int port
     *  @params string protocol
     *  @params Frame  parentFrame
     * Precondition: serviceName is the name of the service (e.g., "ftp")
     *               && port is the port for this service
     *               && protocol is either "TCP" or "UDP"
     *               && parentFrame refers to the owner of this generator
     *                   (for dialogs that need owners)
     * Postcondition: my instance variables have been initialized
     *                 using the parameter values.
     */
    public JavaGenerator(String serviceName, int port, String protocol,
			 Frame parentFrame)
    {
	super(serviceName, port, protocol);

        myCommunicatorBaseName = protocol + "Communicator";
	myParentFrame = parentFrame;
    }

    /** constructor for command-line interfaces
     *  @params string serviceName
     *  @params int port
     *  @params string protocol
     *  @params Frame  parentFrame
     * Precondition: serviceName is the name of the service (e.g., "ftp")
     *               && port is the port for this service
     *               && protocol is either "TCP" or "UDP"
     * Postcondition: my instance variables have been initialized
     *                 using the parameter values.
     */
    public JavaGenerator(String serviceName, int port, String protocol)
    {
	this(serviceName, port, protocol, null);
    }



    /** client source code generating method
     *  Postcondition: the source code has been generated for a client 
     *                  for myService, using myPort and myProtocol
     */
    public void generateClient()
    {
	String parentClassName = this.getProtocol() + "Client";
	String clientName = this.getService() + parentClassName;
	String clientFileName = clientName + ".java";
	String methodStub = clientName + ".interactWithServer()";

	// open up a writer to the client file
	PrintWriter writer = null;
	try
        {	    
	  writer = new PrintWriter( new FileWriter(clientFileName) );
	}
	catch (Exception e) 
	{ 
	    System.err.println("JavaGenerator.generateClient(): " + e); 
	    System.exit(1); 
	}

        // Generate header comment
	writer.println("/** " + clientFileName);
	Date date = new Date();
	writer.println(" *  Auto-generated by Cliser, " + date + ",\n"
		       + " *   except for the body of method:\n"
		       + " *     " + methodStub + ".\n"
		       + " *\n"
		       + " *  See http://cs.calvin.edu/~cliser/ for more details.\n"
		       + " */\n\n"
		       + "import cliser." + parentClassName + ";\n\n"
		       + "class " + clientName + " extends " + parentClassName + "\n"
		       + "{\n"
		       + "  public " + clientName 
		       + "(String remoteHost, int remotePort)\n"
		       + "  {\n"
		       + "    super(\"" + this.getService() 
		       + "\", remoteHost, remotePort);\n"
		       + "  }\n"
		       + "\n\n"
		       + "  public void interactWithServer()\n"
		       + "  {\n"
		       + "    // replace these comments with statements\n"
		       + "    // that define your client's behavior\n"
		       + "  }\n\n"
		       + "  public static void main(String [] args)\n"
		       + "  {\n"
		       + "    " + clientName + " myClient = null;\n\n"
		       + "    if (args.length == 1)\n"
		       + "      myClient = new " + clientName 
		       + "(args[0]," + this.getPort() + ");\n"
		       + "    else if (args.length == 2)\n"
		       + "      myClient = new " + clientName
		       + "( args[0], Integer.parseInt(args[1]) );\n"
		       + "    else\n"
		       + "    {\n"
		       + "      System.err.println(\"\\nUsage: java "
		       + clientName + " <server> [<port>]\\n\");\n"
		       + "      System.exit(1);\n"
		       + "    }\n\n"
		       + "    myClient.run();\n"
		       + "  }\n"
		       + "}\n");

	writer.close();
	
	if (myParentFrame != null)
	    JOptionPane.showMessageDialog(myParentFrame, 
				      "Client written to file: " 
					  + clientFileName,
				      "Client completed...", 
				      JOptionPane.INFORMATION_MESSAGE);
	else
	    System.out.println("JavaGenerator.generateClient(): "
			       + " Client completed and written to file: '"
			       + clientFileName + "'\n\n");
    }

    /** server generator
     *  @params: String concurrentType
     *  Precondition: mySourceLanguage.equals("Java")
     *                 && (concurrentType.equals("Iterative")
     *                     || concurrentType.equals("Concurrent"))
     *  Postcondition: the source code has been generated for a server
     *                  for myService, using myPort, myNetworkInterface,
     *                  myProtocol, and concurrentType.
     */
    public void generateServer(String concurrentType)
    {
	String nameSuffix = this.getProtocol() + "Server";
	String methodStub = null;
	String threadName = this.getService() + "Thread";
	String parentClassName = concurrentType + nameSuffix;
        boolean multithreaded = concurrentType.equals("Concurrent");

	if (multithreaded)
	    methodStub = threadName + ".interactWithClient()";
	else
	    methodStub = this.getService() + parentClassName 
		+ ".interactWithClient()";

	String serverName = this.getService() + parentClassName;
	String serverFileName = serverName + ".java";

	// open up a writer to the server file
	PrintWriter writer = null;
	try
        {	    
	  writer = new PrintWriter( new FileWriter(serverFileName) );
	}
	catch (Exception e) 
	{ 
	    System.err.println("JavaGenerator.generateServer(): " + e); 
	    System.exit(1); 
	}

        // Generate header comment
	writer.println("/** " + serverFileName);
	Date date = new Date();
	writer.println(" *  Auto-generated by Cliser, " + date + ",\n"
		       + " *   except for the body of the method:\n" 
		       + " *     " + methodStub + ".\n"
		       + " *\n"
		       + " *  See http://cs.calvin.edu/~cliser/ for more details.\n"
		       + " */\n\n"
		       + "import cliser." + parentClassName + ";");

	if (multithreaded)   // define performService() within ServiceThread class
        {
	    writer.println("import cliser.ServiceThread;\n\n"
			   + "class " + threadName + " extends ServiceThread\n"
			   + "{\n"
			   + "  public " + threadName + "()\n"
			   + "  {\n"
			   + "    super();\n"
			   + "  }\n\n"
			   + "  public void interactWithClient()\n"
			   + "  {\n"
			   + "    // replace these comments with statements that\n"
			   + "    // define your server-thread's interaction with a client\n"
			   + "  }\n"
			   + "}");

	}

	writer.print("\nclass " + serverName 
		       + " extends " + parentClassName + "\n"
		       + "{\n"
		       + "  public " + serverName + "(int localPort)\n"
		       + "  {\n"
		       + "    super(\"" + this.getService() 
		       + "\", localPort");

	if (multithreaded)  // pass Concurrent superclass a prototype thread
	    writer.print(", new " + threadName + "()");

	writer.println(");\n"
		       + "  }\n");

	if ( !multithreaded ) // define performService() within Server class
	    writer.println("  public void interactWithClient()\n"
			   + "  {\n"
			   + "    // replace these comments with statements that\n"
			   + "    // define your server's interaction with a client\n"
			   + "  }\n"
			   + "\n");

	writer.println("  public static void main(String [] args)\n"
		       + "  {\n"
		       + "    " + serverName + " myServer = null;\n\n"
		       + "    if (args.length == 0)\n"
		       + "      myServer = new " + serverName + "("
		       +         this.getPort() + ");\n"
		       + "    else if (args.length == 1)\n"
		       + "      myServer = new " + serverName
		       + "( Integer.parseInt(args[0]) );\n"
		       + "    else\n"
		       + "    {\n"
		       + "      System.err.println(\"\\nUsage: java "
		       + serverName + " [<port>]\\n\");\n"
		       + "      System.exit(1);\n"
		       + "    }\n\n"
		       + "    myServer.run();\n"
		       + "  }\n"
		       + "}\n");

	writer.close();
	
	if (myParentFrame != null)
	    JOptionPane.showMessageDialog(myParentFrame, 
				      "Server written to file: " 
					  + serverFileName,
				      "Server completed...", 
				      JOptionPane.INFORMATION_MESSAGE);
	else
	    System.out.println("JavaGenerator.generateServer(): "
			       + "Server completed and written to file '"
			       + serverFileName + "'\n\n");
    }



    /** Makefile generating method
     *  Postcondition: error messages saying that Java Makefiles
     *    are not yet supported.
     */
    public void generateMakefile(boolean buildClient, 
				 boolean buildServer,
				 String serverType) 
    {
	if (myParentFrame != null)
	    JOptionPane.showMessageDialog(myParentFrame, 
				      "Makefiles are not yet supported" 
					  + " for Java.",
				      "Makefile Generation", 
				      JOptionPane.INFORMATION_MESSAGE);
	else
	    System.out.println("JavaGenerator.generateServer(): "
			       + "Makefiles are not yet supported for "
			       + "Java\n\n");
    }

    private String myCommunicatorBaseName;
    private Frame  myParentFrame;
}

