/** ServerTCPCommunicator.java
 *  @author Joel C Adams
 *  @version 1.0.  Copyright 2000.  All rights reserved.
 */

package cliser;

import cliser.TCPCommunicator;
import cliser.ClientTCPCommunicator;
import java.net.*;       // ServerSocket

abstract public class ServerTCPCommunicator extends TCPCommunicator
{
   /** explicit constructor
    *  @params port, an int
    *  POST: myServerSocket is a TCP ServerSocket listening at port.
    */
    public ServerTCPCommunicator(int port)
    {
       super("", port);
       try
       {
	   myServerSocket = new ServerSocket(port);
       }
       catch (Exception e)
       {
         System.err.println(e); System.exit(1);
       }
    }

    /** definition of abstract TCPCommunicator.initSocket()
     *  POST: mySocket == null
     *         (mySocket will be set by abstract accept()).
     */
    protected Socket initSocket()
    {
	return null;
    }

    /** abstract method for a server Communicator to accept connections
     * PRE: Some client is going to connect to me
     * @return a TCPCommunicator back to that client
     */
    abstract public TCPCommunicator accept();

    /** myServerSocket accessor
     *  @return myServerSocket
     */
    public final ServerSocket getServerSocket()
    {
	return myServerSocket;
    }

    private ServerSocket myServerSocket;
}
